/*
	File: BLOC.C

	Dumping of blocks (first level)

*/

#include <stdio.h>
#include "b00.h"

#define LG_REGC	0xCD
#define LG_REG		0xCA

/*
  current registration block (EVT file)
	- registration block
	- 3 bytes containing the registration button to light on,
	    the status of the "D" button,...

  registration block (B00  file)
    - registration block for registration # 1
    - registration block for registration # 2
    - registration block for registration # 3
	- ...
    - registration block for registration # 16
	- 84 bytes containing the registration sequence
*/

bloc_regc ( bloc, lg )
unsigned char * bloc;
int lg;
{
	if ( lg != LG_REGC )
		printf ( "Longueur bloc REGC incorrecte: %d au lieu de %d\n",
			lg, LG_REGC );

	bloc_reg1 ( bloc );	/* dump a registration */

	printf ( "Registration courante: 0x%02X 0x%02X 0x%02X\n",
		bloc[LG_REG], bloc[LG_REG+1], bloc[LG_REG+2] );

	return 0;
}

bloc_reg ( bloc, lg )
unsigned char * bloc;
int lg;
{
	int n;
	int nb;

	if ( lg != LG_REG*16 + 84 )
		printf ( "Longueur bloc REG incorrecte: %d au lieu de %d\n",
			lg, LG_REG );

	nb = 16;
	if ( model == MODEL_EL40 )
		nb = 8;
	if ( opt & OPT_REG8 )
		nb = 8;
	if ( opt & OPT_REG16 )
		nb = 16;

	for ( n = 0; n < nb; n++ )
	{
		printf ( "\nREGISTRATION %2d:\n", n+1 );
		bloc_reg1 ( bloc + LG_REG * n );	/* dump a registration */
	}

	regseq ( bloc + LG_REG * 16 );	/* dump the registration sequence */

	return 0;
}

#define LG_VOI		0x1CC

bloc_voi ( bloc, lg )
unsigned char * bloc;
int lg;
{
	int pt;
	int n;

	n = 1;
	for ( pt = 0; pt + LG_VOI <= lg ; pt += LG_VOI )
	{
		printf ( "\nUSER VOICE %2d:", n++ );
		bloc_voi1 ( bloc + pt, LG_VOI );

		if ( opt & OPT_USER4 )
			if ( n > 4 )
				return 0;
	}

	if ( pt != lg )
	{
		printf ( "\nUSER VOICE %2d:", n++ );
		printf ( "Erreur: longueur incorrecte: %d\n", lg - pt );
		return 1;	
	}

	return 0;
}

#define LG_FLU ( 8*8 + 0x40 + 8*8 + 0x40 )

bloc_flu ( bloc, lg )
unsigned char * bloc;
int lg;
{
	int i;
	char titre[20];

	if ( lg != LG_FLU )
		printf ( "Longueur bloc FLUTE incorrecte: %d au lieu de %d\n",
			lg, LG_FLU );

	if ( opt & OPT_FLU )
	{
		for ( i = 0; i < 8; i++ )
		{
			printf ( "\n" );
			sprintf ( titre, "USER FLUTE %d", i + 1 );
			flute ( titre, bloc + 8*i );
		}

		printf ( "\nVOICE 1:" );
		bloc_voi1 ( bloc + 0x40, 0x40 );

		for ( i = 0; i < 8; i++ )
		{
			printf ( "\n" );
			sprintf ( titre, "USER FLUTE %d", i + 1 + 8 );
			flute ( titre, bloc + 0x80 + 8*i );
		}

		printf ( "\nVOICE 2:" );
		bloc_voi1 ( bloc + 0xc0, 0x40 );
	}

	return 0;
}

#define LG_PAT ( 40 * ( 16 + 5 ) )

bloc_pat ( bloc, lg )
unsigned char * bloc;
int lg;
{
	int lgmes;
	int i;

	if ( lg < LG_PAT + 2 )
		printf ( "Longueur bloc PAT incorrecte: %d au lieu de + de %d\n",
			lg, LG_PAT );

	printf ( "\nUSER PATTERN:\n" );
	pattern ( bloc );

	lgmes = bloc[LG_PAT] + 256 * bloc[ LG_PAT + 1 ];

	yy ( "YY10", bloc + LG_PAT + 2, 0x28 );

	printf ( "\nMESURES:\n" );
	mesur ( bloc + LG_PAT + 2 + 0x28 );

	if ( opt & OPT_OUT )
	{
		for ( i = 0; i < 4; i++ )
			outseq ( i, bloc + LG_PAT + 2 + 0x28 );
	}

	if ( opt & OPT_PAT )
	{
		outpat ( bloc + LG_PAT + 2 + 0x28 );
	}

	yy ( "YY11", bloc + lg - 2 , 2 );

	return 0;
}

#define LG_SEQ ( 4 * 120 * 2 )

bloc_seq ( bloc, lg )
unsigned char * bloc;
int lg;
{
	int i;

	if ( lg != LG_SEQ )
		printf ( "Longueur bloc SEQUENCE incorrecte: %d au lieu de %d\n",
			lg, LG_SEQ );

	for ( i = 0; i < 4; i++ )
	{
		sequence ( i, bloc + ( i*240 ) );
	}

	return 0;
}

