/*
	File: CODE.C

	Conversion of numeric codes to readable text strings
*/

#include <stdio.h>

#include "b00.h"

char * modele ( c )
unsigned char c;
{
	static char m[10];

	switch ( c & 0x7f )
	{
		case MODEL_EL40:	return "EL40";
		case MODEL_EL60:	return "EL60";
		case MODEL_EL70:	return "EL70";
		case MODEL_EL90:	return "EL90";
	}
	sprintf ( m, "0x%02X", c );
	return m;
}

char * clavier ( c )
int c;
{
	switch ( c )
	{
		case CLAV_LEAD:  return "LEAD";
		case CLAV_UK1 :  return "UK1";
		case CLAV_UK2 :  return "UK2";
		case CLAV_LK1 :  return "LK1";
		case CLAV_LK2 :  return "LK2";
		case CLAV_PED1:  return "PED1";
		case CLAV_PED2:  return "PED2";
		case CLAV_UK  :  return "UK";
		case CLAV_LK  :  return "LK";
		case CLAV_PD  :  return "PD";
	}
	return "????";
}

int volume ( code )
unsigned char code;
{
	static unsigned tab_vol[25] =
	{
		0x7f,  0x2c,  0x28,  0x24,  0x20,
		0x1d,  0x1a,  0x17,  0x14,  0x12,
		0x10,  0x0e,  0x0c,  0x0b,  0x0a,
		0x09,  0x08,  0x07,  0x06,  0x05,
		0x04,  0x03,  0x02,  0x01,  0x00
	};
	int vol;

	for ( vol = 0; vol <= 24; vol++ )
	{
		if ( ( code & 0x7f ) >= tab_vol [ vol ] )
			break;
	}
	return vol;
}

char * feet ( code )
unsigned char code;
{
	static char strfeet[10];

	switch ( code & 0x07 )
	{
		case 0:	return "";
		case 1:	return " 16'";
		case 2:	return " 8'";
		case 3:	return " 4'";
		default:
			sprintf ( strfeet, " ?%02d", code );
			return strfeet;
	}
}

char * effet ( code )
unsigned char code;
{
	static char streffet[10];

	switch ( code & 0x07 )
	{
		case 0:	return "";
		case 1:	return " OFF";
		case 2:	return " TRM";
		case 3:	return " CEL";
		case 4:	return " DELAY";
		default:
			sprintf ( streffet, " ?%02d", code );
			return streffet;
	}
}

char * tone ( code )
unsigned char code;
{
	static char strtone[10];

	if ( code == 0 )
		return "";

	if ( code & 0x04 )
		sprintf ( strtone, " -%d", ( code & 0x03 ) );
	else
		sprintf ( strtone, " +%d", ( code & 0x03 ) );
	return strtone;
}

char * uvib ( del, spe, dep )
unsigned char del, spe, dep;
{
	static char struvib[30];

	sprintf ( struvib, " U.VIB=dl%d,sp%d,dp%d",
		del & 0x07, spe & 0x07, dep & 0x07 );
	return struvib;
}

char * tab_ryth[] =
{
/* [00] B */
			"March1", "March2", "March3",
			"Polka1", "Polka2", "Country1", "Country2",
			"Broadway", "Baroque",
/* [09] A */
			"Waltz1", "Waltz2", "Waltz3",
			"Waltz4", "Waltz5",
			"J.Waltz1", "J.Waltz2", "J.Waltz3", "Bolero",
/* [18] 9 */
			"Swing1", "Swing2", "Swing3", "Swing4",
			"Swing5",
			"Dixie1", "Dixie2",
			"J.Ballad",
/* [26] 8 */
			"Bounce1", "Bounce2", "Bounce3",
/* [29] F */
			"Reggae1", "Reggae2",
/* [31] 7 */
			"S.Rock1", "S.Rock2", "S.Rock3",
/* [34] 3 */
			"8Beat1", "8Beat2", "8Beat3",
			"8Beat4", "8Beat5",
/* [39] ? */
			"Swing6",
			"D.Pop1", "D.Pop2", "D.Pop3", "D.Pop4",
/* [44] 6 */
			"Tango1", "Tango2", "Tango3",
/* [47] E */
			"Mambo", "Salsa",
/* [49] 5 */
			"Chacha",
/* [50] D */
			"Rhumba", "Beguine",
/* [52] 4 */
			"Samba1", "Samba2", "Samba3",
/* [55] C */
			"Bossa1", "Bossa2", "Bossa3",
/* [58] 2 */
			"16Beat1", "16Beat2", "16Beat3",
			"16Beat4", "16Beat5",
			"Funk1", "Funk2", "Funk3",
/* [66] */	"??66??",
	"User-1A", "User-1B", "User-1C", "User-1D",
	"User-2A", "User-2B", "User-2C", "User-2D",
	"User-3A", "User-3B", "User-3C", "User-3D",
	"User-4A", "User-4B", "User-4C", "User-4D",
	"User-5A", "User-5B", "User-5C", "User-5D",
	"User-6A", "User-6B", "User-6C", "User-6D",
	"User-7A", "User-7B", "User-7C", "User-7D",
	"User-8A", "User-8B", "User-8C", "User-8D",
	"User-1F", "User-2F", "User-3F", "User-4F",
	"User-5F", "User-6F", "User-7F", "User-8F",
};

char * rythme ( code )
unsigned char code;
{
	static char strryth[30];

	if ( ( code & 0x7f ) < sizeof ( tab_ryth ) / sizeof ( tab_ryth[0] ) )
	{
		return tab_ryth[ code & 0x7f ];
	}

	sprintf ( strryth, "Rythme_%03d", code & 0x7f );
	return strryth;
}

struct struct_inst
{
	unsigned char code;
	char * inst;
} tab_inst[] =
{
	{ 1,"String1"  },{ 2,"Violin"   },{ 3,"Cont.Bass1"},{ 4,"Brass1"   },
	{ 5,"SynBrass1"},{ 6,"Horn1"    },{ 7,"Trumpet1" },{ 8,"Tuba"     },
	{ 9,"Flute"    },{10,"Oboe"     },{11,"Clarinet" },{12,"Saxo1"    },
	{13,"Tutti1"   },{14,"Chorus"   },{15,"Harmonica"},{16,"Organ"    },
	{17,"OrgBass1" },{18,"Piano"    },{19,"Guitar1"  },{20,"Vibrphon" },
	{21,"E.Bass1"  },{22,"SynBass2" },{23,"Cosmic1"  },{24,"Cosmic2"  },
	{25,"Cosmic3"  },{26,"String2"  },{27,"PizzStr"  },{28,"PizzBass" },
/*	{29,""         },{30,""         },*/
	{31,"Trombone" },{32,"Timpani"  },{33,"Whistle"  },{34,"Bassoon"  },
	{35,"Tutti2"   },{36,"Tutti3"   },
	{37,"PopOrgan" },{38,"JazzOrgan"},{39,"Accordio" },{40,"OrgBass2" },
	{41,"E.Piano"  },{42,"Harpsi"   },{43,"E.Guitar" },{44,"Guitar2"  },
	{45,"Harp"     },{46,"Glocken"  },
/*	{47,""         },*/
	{48,"Marimba"  },{49,"E.Bass2"},{50,"Cosmic4" },
	{52,"String3"  },{53,"SynStr1"  },
	{57,"Brass2"   },{58,"Brass3"   },{59,"MutedTrp" },
	{77,"Xylophone"},{78,"SynBass"  },
	{80,"Cosmic6"  },
	{82,"SynStr2"  },{83,"String5"  },{84,"Violin3"  },
	{88,"Brass4"   },{89,"SynBrass2"},
	{91,"MutedHorn"},{92,"Trumpet3" },{93,"Trumpet4" },
	{96,"MutedTrb" },{97,"Timp.Roll"},
	{101,"Oboe2"   },
	{105,"Saxophon2"},
	{107,"Tutti6"  },
	{109,"Chorus3" },
	{117,"JazzOrgan3"},
	{123,"DistGuit"},
	{120,"HonkyTonk"},
	{130,"SynBass3"},
/*	{,""   }, */
	{224+ 0,"User1" },{224+ 1,"User2" },{224+ 2,"User3" },{224+ 3,"User4" },
	{224+ 4,"User5" },{224+ 5,"User6" },{224+ 6,"User7" },{224+ 7,"User8" },
	{224+ 8,"User9" },{224+ 9,"User10"},{224+10,"User11"},{224+11,"User12"},
	{224+12,"User13"},{224+13,"User14"},{224+14,"User15"},{224+15,"User16"},
};

char * instrument ( code )
unsigned char code;
{
	static char strinst[20];
	int i;

	for ( i = 0; i < sizeof ( tab_inst ) / sizeof ( tab_inst[0] ); i++ )
	{
		if ( tab_inst[i].code == code )
			return tab_inst[i].inst;
	}
	sprintf ( strinst, "Instr_%03d", code );
	return strinst;
}

int yy ( n, pt, lg )
char * n;
unsigned char *pt;
int lg;
{
	int i;

	if ( opt & OPT_YY )
	{
		printf ( "\n%6.6s=", n );
		for ( i = 0; i < lg; i++ )
		{
			printf ( " %02X", pt[i] );
			if ( i % 16 == 15 )
				printf ( "\n%6.6s+", n );
		}
		printf ( "\n" );
	}
	return 0;
}

int xx ( txt, pt, lg )
char * txt;
unsigned char * pt;
int lg;
{
	int i;

	if ( opt & OPT_XX )
	{
		printf ( "%s:", txt );
		for ( i = 0; i < lg; i++ )
			printf ( " X%d=%02X", i, pt[i] );
		printf ( "\n" );
	}

	return 0;
}

