/*
	File: FIC.C

	Reads B00 file (BULK DUMP DATA) or "current registration" files
	generated by the EVT.EXE program

*/

#include <stdio.h>
#include <string.h>
#include <alloc.h>

#include "b00.h"

FILE * file;
char * fichier;	/* name of the file treated, path removed, extension removed */
int model;		/* Electone modell code */

int fic ( fich )
char * fich;
{
	unsigned char entete[10];
	char *p;
	int ret;

	if ( strcmp ( fich, "-" ) == 0 )	/* read data on standard input */
	{
		file = stdin;
		fichier = "stdin";
	}
	else
	{
		file = fopen ( fich, "rb" );

		fichier = fich;
		for ( p = fichier; ( *p != '\0' ) && ( *p != '.' ) ; p++ )
			if ( ( *p == '/' ) || ( *p == '\\' ) )
				fichier = p + 1;
		*p = 0;
	}

	printf ( "FILE: %s\n", fich );

	if ( file == NULL )
	{
		printf ( "Error while opening the file.\n" );
		return 1;
	}

	/* read the 6 bytes header of the file */
	ret = fread ( entete, 1, 6, file );
	if ( ret != 6 )
	{
		fprintf ( stderr, "Error while reading file header.\n" );
		return 1;
	}

	xx ( "HEADER", entete, 6 );

	if ( ( entete[0] != 0xf0 )
	  || ( entete[1] != 0x43 )
	  || ( entete[2] != 0x70 )
	  || ( ( entete[3] != 0x70 ) && ( entete[3] != 0x78 ) ) )
	{
		printf ( "Incorrect header.\n" );
		return 1;
	}

	model = entete[5];

	switch ( entete[4] )
	{
		case 0x0:	/* BULK DUMP DATA */
			printf ( "HEADER: Bulk Dump Data %s\n", modele(model) );
			bulk ( );
			break;

		case 0x42:	/* CURRENT REGISTRATION */
			printf ( "HEADER: Current Registration %s\n", modele(model) );
			curreg ( );
			break;

		default:
			printf ( "HEADER: unknown type\n" );
			return 1;
	}

	printf ( "\n" );

	return 0;
}

int bulk ( )
{
	unsigned char entete[6];
	int type;
	int lg;
	int ret;
	int i;
	int chk;
	unsigned char * bloc;

	do
	{
		ret = fread ( entete, 1, 6, file );		/* read 6 bytes header */
		if ( ret <= 0 )
		{
			printf ( "Read error.\n" );
			return 1;
		}

		if ( entete[0] == 0xF7 )	/* F7 is the end of the SYSEX message */
			break;

		if ( ret != 6 )
		{
			printf ( "Read error.\n" );
			return 1;
		}

		xx ( "BLOC", entete, 6 );

		if ( ( entete[4] | entete[5] ) != 0 )
		{
			printf ( "Incorrect Header.\n" );
			return 1;
		}

		type = entete[0];
		lg = (int)entete[2] + 128 * (int)entete[3];

		bloc = (unsigned char *) malloc ( lg + 1 );
		if ( bloc == NULL )
		{
			printf ( "Erreur memoire (%d octets).\n", lg+1 );
			return 1;
		}

		ret = fread ( bloc, 1, lg + 1, file );	/* read the block and the checksum */
		if ( ret != lg+1 )
		{
			printf ( "Erreur lecture bloc.\n" );
			return 1;
		}

		chk = 0;					/* check the checksum */
		for ( i = 0; i < lg; i++ )	/* compute sum of data block bytes */
			chk = ( chk + bloc[i] ) & 0x7F;
		if ( chk + bloc[lg] != 0x7f )	/* checksum is correct ? */
		{
			printf ( "Checksum incorrect: offset 0x%lX, 0x%02X au lieu de 0x%02X.\n",
				ftell ( file ) -1, bloc[lg], 0x7f - chk );
		}

		switch ( type )		/* dump the data block */
		{
			case 0:
				bloc_reg ( bloc, lg );
				break;

			case 1:
				bloc_voi ( bloc, lg );
				break;

			case 2:
				bloc_flu ( bloc, lg );
				break;

			case 3:
				bloc_pat ( bloc, lg );
				break;

			case 4:
				bloc_seq ( bloc, lg );
				break;

			default:
				printf ( "Bloc de type %d inconnu.\n", type );
				break;
		}

		free ( bloc );

	}
	while ( 1 );

	return 0;
}

curreg ( )
{
	unsigned char entete[3], *bloc, c;
	int ret;
	int chk;
	int i;
	int lg;

	/* 2 bytes header containing the data length */
	ret = fread ( entete, 1, 2, file );
	if ( ret != 2 )
	{
		printf ( "Format entete incorrect.\n" );
		return 1;
	}

	lg = (int)entete[0] + 128 * (int)entete[1];
	printf ( "lg=%d\n", lg );

	/* read the data block, the checksum byte and the F7 byte */
	bloc = (unsigned char *) malloc ( lg+2 );
	if ( bloc == NULL )
	{
		printf ( "Erreur memoire (%d octets).\n", lg+2 );
		return 1;
	}

	i = 0;
	while ( i < lg )
	{
		/* read the file 1 byte at a time */
		ret = fread ( &c, 1, 1, file );
		if ( ret != 1 )
		{
			printf ( "Erreur lecture bloc.\n" );
			return 1;
		}

		/* in this file, the data bytes greater than 64 (40 in hex)
		   are coded as two bytes:
		   byte to encode: B7  B6  B5  B4  B3  B2  B1  B0
		   1st coded byte:  1   1   B5  B4  B3  B2  B1  B0
		   2nd coded byte:  0   0   B7  B6   0   0   0   0
		*/

		if ( c & 0xc0 )		/* is it a 2-bytes code ? */
		{
			bloc[i] = c & 0x3f;		/* extract the first part */
			ret = fread ( &c, 1, 1, file );
			if ( ret != 1 )
			{
				printf ( "Erreur lecture bloc.\n" );
				return 1;
			}
			bloc[i++] |= ( c & 0x30 ) << 2;	/* complete with second part */
		}
		else	/* no, the byte is < 64 */
		{
			bloc[i++] = c;
		}
	}

	/* read checksum and F7 */
	ret = fread ( &bloc[i], 1, 2, file );
	if ( ret != 2 )
	{
		printf ( "Erreur lecture bloc.\n" );
		return 1;
	}

	if ( bloc[lg+1] != 0xF7 )
	{
		printf ( "Erreur pas de fin de bloc.\n" );
	}

	chk = 0;	/* check the checksum */
	for ( i = 0; i < lg; i++ )
		chk = ( chk + bloc[i] ) & 0x7F;
	if ( chk + bloc[lg] != 0x7f )
	{
		printf ( "Checksum incorrect: 0x%02X au lieu de 0x%02X.\n",
			bloc[lg], 0x7f - chk );
	}

	bloc_regc ( bloc, lg );		/* dump the block */

	free ( bloc );

	return 0;
}

