#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <mevt.h>

/* Liste des codes
FF/01   commentaire
FF/03   nom piste
FF/20   canal
FF/21   port
FF/2F   fin de piste
FF/51   tempo
FF/54   ????
FF/58   mesure
FF/59   cl

80      note off
90      note on
* le code 80 ou 90 peut tre saut si identique au prcdent

B0      controlleur
C0      program change (patch)
D0      after touch canal
F0      exclusive

*/

long miw_trk_lg_pos;
long miw_trk_nb_pos;
int miw_trk_nb;

long miw_time, miw_time_w; /* temps en nombre de clic */

int miw_time_byte ( LONG t, BYTE mask )
{
    if ( t > 0x7f )
    {
        miw_time_byte ( t >> 7, 0x80 );
    }
    mf_write_B ( ( t & 0x7f ) | mask );
    
    return 0;
}

int miw_time_sep ( )
{
    miw_time_byte ( miw_time - miw_time_w, 0x00 );
    miw_time_w = miw_time;
    
    return 0;
}

int miw_time_set ( long t )
{
    miw_time = t;
	if ( miw_time_w > miw_time )
		miw_time_w = miw_time;
    return 0;
}

int miw_time_plus ( long t )
{
    miw_time += t;
    
    return 0;
}

int miw_time_reset ( )
{
    miw_time = 0L;
    miw_time_w = 0L;
    
    return 0;
}

int miw_header ( )
{
    miw_time_reset();
    mf_write_S ( "MThd" );
    mf_write_L ( 6 );

    /* type de fichier MIDI:
        0= une seule piste
        1=piste chef d'orchestre + n pistes */
    mf_write_W ( 1 );

    /* nombre de pistes, mis  jour  la fin */
    miw_trk_nb_pos = mf_pos();
    mf_write_W ( 0 );
    
    mf_write_W ( CLIC );

    return 0;
}

int miw_comment ( char *x )
{
    miw_time_sep ( );
    mf_write_B ( 0xff );
    mf_write_B ( 0x01 );
    mf_write_B ( strlen(x) );
    mf_write_S ( x );
    
    return 0;
}

int miw_trk_name ( char *x )
{
    miw_time_sep ( );
    mf_write_B ( 0xff );
    mf_write_B ( 0x03 );
    mf_write_B ( strlen(x) );
    mf_write_S ( x );
    
    return 0;
}

int miw_tempo ( WORD tempo )
{
    miw_time_sep ( );
    mf_write_B ( 0xff );
    mf_write_B ( 0x51 );
    mf_write_B ( 3 );
    /* convertir de "nbre de noires par minute"
       en "nbre de s par noires */
    mf_write_3 ( (60L*1000000L)/(LONG)tempo );
    
    return 0;
}

int miw_mesure ( BYTE n, BYTE d )
/* n = numrateur
   d = dnominateur (noire = 4) */
{
    BYTE val_d, code_d;

    for ( val_d=1, code_d=0; val_d < d; val_d <<= 1, code_d++ )
        ;

    miw_time_sep ( );
    mf_write_B ( 0xff );
    mf_write_B ( 0x58 );
    mf_write_B ( 4 );
    mf_write_B ( n );
    mf_write_B ( code_d );
    mf_write_B ( ((4*24)/d) );    /* 1 noire = 24 ticks, ?? */
    mf_write_B ( 0x08 );
    
    return 0;
}

int miw_clef ( BYTE c )
{
    /*
        F = -1
        C = 0
        G = 1
        A = 2 etc
    */
    miw_time_sep ( );
    mf_write_B ( 0xff );
    mf_write_B ( 0x59 );
    mf_write_B ( 2 );
    mf_write_B ( c );
    mf_write_B ( 0x00 );
    
    return 0;
}

int miw_port ( BYTE p )
/* port midi de 0  ... */
{
    miw_time_sep ( );
    mf_write_B ( 0xff );
    mf_write_B ( 0x21 );
    mf_write_B ( 1 );
    mf_write_B ( p );
    return 0;
}

int miw_canal ( BYTE canal )
{
    miw_time_sep ( );
    mf_write_B ( 0xff );
    mf_write_B ( 0x20 );
    mf_write_B ( 1 );
    mf_write_B ( canal-1 );
    return 0;
}

int miw_noteon ( BYTE canal, BYTE note, BYTE velo )
{
    miw_time_sep ( );
    mf_write_B ( 0x90 + canal - 1 );
    mf_write_B ( note );                /* C4 = 0x30 */
    mf_write_B ( velo & 0x7F );
    return 0;
}

int miw_noteoff ( BYTE canal, BYTE note, BYTE velo )
{
    miw_time_sep ( );
    mf_write_B ( 0x80 + canal - 1 );
    mf_write_B ( note );
    mf_write_B ( velo & 0x7f );
    return 0;
}

int miw_controller ( BYTE canal, BYTE control, BYTE valeur )
{
	miw_time_sep ( );
    mf_write_B ( 0xB0 + canal - 1 );
    mf_write_B ( control );
    mf_write_B ( valeur & 0x7f );
    return 0;
}

int miw_progchange ( BYTE canal, BYTE prog )
{
    miw_time_sep ( );
    mf_write_B ( 0xC0 + canal - 1 );
    mf_write_B ( prog & 0x7f );
    return 0;
}

int miw_aftertouch ( BYTE canal, BYTE valeur )
{
    miw_time_sep ( );
    mf_write_B ( 0xD0 + canal - 1 );
    mf_write_B ( valeur & 0x7f );
    return 0;
}

int miw_exclusif ( BYTE *data, int lg )
{
    BYTE *data1;
    int lg1;
    
    miw_time_sep ( );

    if ( data[0] != 0xf0 )
    {
        data1 = data;
        lg1 = lg;
    }
    else
    {
        data1 = data+1;
        lg1 = lg-1;
    }

    if ( data1[lg1-1] == 0xf7 )
    {
        lg1 = lg1-1;
    }
    
    mf_write_B ( 0xF0 );
    if ( lg1+1 > 127 )
	{
		mf_write_B ( ( ( lg1+1 ) >> 7 ) | 0x80 );
		mf_write_B ( ( lg1+1 ) & 0x7F );
	}
	else
	{
		mf_write_B ( lg1+1 );
	}
    mf_write ( data1, lg1 );
    mf_write_B ( 0xF7 );

    return 0;
}

int miw_open_trk ( )
{
    miw_trk_nb++;
    miw_time_reset ( );
    mf_write_S ( "MTrk" );
    miw_trk_lg_pos = mf_pos ( );
    mf_write_L ( 0L );

    return 0;
}

int miw_close_trk ( )
{
    long pos;

    miw_time_sep ( );
    mf_write_B ( 0xff );
    mf_write_B ( 0x2f );
    mf_write_B ( 0 );

    pos = mf_pos ( );
    mf_set ( miw_trk_lg_pos );
    mf_write_L ( pos - miw_trk_lg_pos - 4 );
    mf_set ( pos );
    
    return 0;
}

int miw_open ( char *nom )
{
    mf_w_open ( nom );
    miw_header ( );
    miw_time_reset ( );
    miw_trk_nb = 0;
    return 0;
}

int miw_close ( )
{
    long pos;
    
    pos = mf_pos();
    mf_set ( miw_trk_nb_pos );
    mf_write_W ( miw_trk_nb );
    mf_set ( pos );
    mf_close ( );
    
    return 0;
}

